@@ -5,7 +5,7 @@ from dataclasses import dataclass, field
 from functools import partial
 from inspect import Parameter, getmro, isclass
 from inspect import signature as inspect_signature
-from typing import Any, Protocol, Self, overload, runtime_checkable
+from typing import TYPE_CHECKING, Any, Protocol, Self, overload, runtime_checkable
 
 import injection
 
@@ -89,58 +89,72 @@ class HandlerDecorator[I, O]:
     manager: HandlerManager[I, O]
     injection_module: injection.Module = field(default_factory=injection.mod)
 
-    @overload
-    def __call__(
-        self,
-        input_or_handler_type: type[I],
-        /,
-    ) -> Callable[[HandlerType[[I], O]], HandlerType[[I], O]]: ...
+    if TYPE_CHECKING:  # pragma: no cover
 
-    @overload
-    def __call__(
-        self,
-        input_or_handler_type: HandlerType[[I], O],
-        /,
-    ) -> HandlerType[[I], O]: ...
+        @overload
+        def __call__(
+            self,
+            input_or_handler_type: type[I],
+            /,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> Callable[[HandlerType[[I], O]], HandlerType[[I], O]]: ...
 
-    @overload
-    def __call__(
-        self,
-        input_or_handler_type: None = ...,
-        /,
-    ) -> Callable[[HandlerType[[I], O]], HandlerType[[I], O]]: ...
+        @overload
+        def __call__(
+            self,
+            input_or_handler_type: HandlerType[[I], O],
+            /,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> HandlerType[[I], O]: ...
+
+        @overload
+        def __call__(
+            self,
+            input_or_handler_type: None = ...,
+            /,
+            *,
+            threadsafe: bool | None = ...,
+        ) -> Callable[[HandlerType[[I], O]], HandlerType[[I], O]]: ...
 
     def __call__(
         self,
         input_or_handler_type: type[I] | HandlerType[[I], O] | None = None,
         /,
+        *,
+        threadsafe: bool | None = None,
     ) -> Any:
-        if input_or_handler_type is None:
-            return self.__decorator
-
-        elif isclass(input_or_handler_type) and issubclass(
-            input_or_handler_type,
-            Handler,
+        if (
+            input_or_handler_type is not None
+            and isclass(input_or_handler_type)
+            and issubclass(input_or_handler_type, Handler)
         ):
-            return self.__decorator(input_or_handler_type)
+            return self.__decorator(input_or_handler_type, threadsafe=threadsafe)
 
-        return partial(self.__decorator, input_type=input_or_handler_type)  # type: ignore[arg-type]
+        return partial(
+            self.__decorator,
+            input_type=input_or_handler_type,  # type: ignore[arg-type]
+            threadsafe=threadsafe,
+        )
 
     def __decorator(
         self,
         wrapped: HandlerType[[I], O],
+        /,
         *,
         input_type: type[I] | None = None,
+        threadsafe: bool | None = None,
     ) -> HandlerType[[I], O]:
-        factory = self.injection_module.make_async_factory(wrapped)
+        factory = self.injection_module.make_async_factory(wrapped, threadsafe)
         input_type = input_type or _resolve_input_type(wrapped)
         self.manager.subscribe(input_type, factory)
         return wrapped
 
 
 def _resolve_input_type[I, O](handler_type: HandlerType[[I], O]) -> type[I]:
-    fake_handle_method = handler_type.handle.__get__(NotImplemented)
-    signature = inspect_signature(fake_handle_method, eval_str=True)
+    fake_method = handler_type.handle.__get__(NotImplemented, handler_type)
+    signature = inspect_signature(fake_method, eval_str=True)
 
     for parameter in signature.parameters.values():
         input_type = parameter.annotation
