@@ -1,7 +1,8 @@
-from typing import Any
+from typing import Any, Final
 
 import injection
 
+from cq._core.dispatcher.base import Dispatcher
 from cq._core.dispatcher.bus import Bus, SimpleBus, TaskBus
 from cq._core.handler import (
     HandlerDecorator,
@@ -15,40 +16,56 @@ Command = object
 Event = object
 Query = object
 
-type CommandBus[T] = Bus[Command, T]
-type EventBus = Bus[Event, None]
-type QueryBus[T] = Bus[Query, T]
+type CommandBus[T] = Dispatcher[Command, T]
+type EventBus = Dispatcher[Event, None]
+type QueryBus[T] = Dispatcher[Query, T]
 
 AnyCommandBus = CommandBus[Any]
 
 
-command_handler: HandlerDecorator[Command, Any] = HandlerDecorator(
+command_handler: Final[HandlerDecorator[Command, Any]] = HandlerDecorator(
     SingleHandlerManager(),
 )
-event_handler: HandlerDecorator[Event, None] = HandlerDecorator(
+event_handler: Final[HandlerDecorator[Event, None]] = HandlerDecorator(
     MultipleHandlerManager(),
 )
-query_handler: HandlerDecorator[Query, Any] = HandlerDecorator(
+query_handler: Final[HandlerDecorator[Query, Any]] = HandlerDecorator(
     SingleHandlerManager(),
 )
 
 
-@injection.injectable(inject=False, mode="fallback")
-def new_command_bus() -> CommandBus:  # type: ignore[type-arg]
+def new_command_bus(*, threadsafe: bool | None = None) -> Bus[Command, Any]:
     bus = SimpleBus(command_handler.manager)
     transaction_scope_middleware = InjectionScopeMiddleware(
         CQScope.TRANSACTION,
         exist_ok=True,
+        threadsafe=threadsafe,
     )
     bus.add_middlewares(transaction_scope_middleware)
     return bus
 
 
-@injection.injectable(inject=False, mode="fallback")
-def new_event_bus() -> EventBus:
+def new_event_bus() -> Bus[Event, None]:
     return TaskBus(event_handler.manager)
 
 
-@injection.injectable(inject=False, mode="fallback")
-def new_query_bus() -> QueryBus:  # type: ignore[type-arg]
+def new_query_bus() -> Bus[Query, Any]:
     return SimpleBus(query_handler.manager)
+
+
+@injection.injectable(inject=False, mode="fallback")
+def _() -> CommandBus:  # type: ignore[type-arg]
+    return new_command_bus()
+
+
+@injection.injectable(inject=False, mode="fallback")
+def _() -> EventBus:
+    return new_event_bus()
+
+
+@injection.injectable(inject=False, mode="fallback")
+def _() -> QueryBus:  # type: ignore[type-arg]
+    return new_query_bus()
+
+
+del _
